@@ -99,20 +99,20 @@
 #' @method as.mcmc emmGrid
 #' @export as.mcmc.emmGrid
 #' @examples
-#' require("coda")
-#'
-#' ### A saved reference grid for a mixed logistic model (see lme4::cbpp)
-#' cbpp.rg <- do.call(emmobj, 
+#' if(requireNamespace("coda")) {
+#'   ### A saved reference grid for a mixed logistic model (see lme4::cbpp)
+#'   cbpp.rg <- do.call(emmobj, 
 #'     readRDS(system.file("extdata", "cbpplist", package = "emmeans")))
-#' # Predictive distribution for herds of size 20
-#' # (perhaps a bias adjustment should be applied; see "sophisticated" vignette)
-#' pred.incidence <- as.mcmc(regrid(cbpp.rg), likelihood = "binomial", trials = 20)
+#'   # Predictive distribution for herds of size 20
+#'   # (perhaps a bias adjustment should be applied; see "sophisticated" vignette)
+#'   pred.incidence <- coda::as.mcmc(regrid(cbpp.rg), likelihood = "binomial", trials = 20)
+#' }
 as.mcmc.emmGrid = function(x, names = TRUE, sep.chains = TRUE, 
                            likelihood, NE.include = FALSE, ...) {
     if (is.na(x@post.beta[1])) {
         stop("No posterior sample -- can't make an 'mcmc' object")
     }
-# notes on estimabilityn issues:
+# notes on estimability issues:
 # 1. Use @bhat to determine which coefs to use
 # 2. @nabasis as in freq models
 # 3. @post.beta we will EXCLUDE cols corresp to NAs in @bhat
@@ -529,12 +529,6 @@ recover_data.stanreg = function(object, ...) {
 
 # note: mode and rescale are ignored for some models
 emm_basis.stanreg = function(object, trms, xlev, grid, mode, rescale, ...) {
-    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
-    if(is.null(contr <- object$contrasts))
-        contr = attr(model.matrix(object), "contrasts")
-    X = model.matrix(trms, m, contrasts.arg = contr)
-    bhat = rstanarm::fixef(object)
-    V = vcov(object)
     misc = list()
     if (!is.null(object$family)) {
         if (is.character(object$family)) # work around bug for stan_polr
@@ -542,6 +536,25 @@ emm_basis.stanreg = function(object, trms, xlev, grid, mode, rescale, ...) {
         else
             misc = .std.link.labels(object$family, misc)
     }
+    # Previous code...
+    ### m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
+    ### if(is.null(contr <- object$contrasts))
+    ###     contr = attr(model.matrix(object), "contrasts")
+    ### X = model.matrix(trms, m, contrasts.arg = contr)
+    ### bhat = rstanarm::fixef(object)
+    ### nms = intersect(colnames(X), names(bhat))
+    ### bhat = bhat[nms]
+    ### V = vcov(object)[nms, nms, drop = FALSE]
+
+    # Instead, use internal routine in rstanarm to get the model matrix
+    # Later, we'll get bhat and V from the posterior sample because
+    # the vcov(object) doesn't always jibe with fixef(object)
+    if(is.null(object$contrasts)) # old version of rstanarm where contrasts may get lost.
+        object$contrasts = attr(model.matrix(object), "contrasts")
+    pp_data = get("pp_data", envir = getNamespace("rstanarm"))
+    X = pp_data(object, newdata = grid, re.form = ~0, ...)[[1]]
+    nms = colnames(X)
+    
     if(!is.null(object$zeta)) {   # Polytomous regression model
         if (missing(mode))
             mode = "latent"
@@ -549,19 +562,19 @@ emm_basis.stanreg = function(object, trms, xlev, grid, mode, rescale, ...) {
             mode = match.arg(mode, 
                              c("latent", "linear.predictor", "cum.prob", "exc.prob", "prob", "mean.class"))
         
-        xint = match("(Intercept)", colnames(X), nomatch = 0L)
+        xint = match("(Intercept)", nms, nomatch = 0L)
         if (xint > 0L) 
             X = X[, -xint, drop = FALSE]
         k = length(object$zeta)
         if (mode == "latent") {
-            if (missing(rescale)) 
+            ### if (missing(rescale))  #------ (Disabl rescale)
                 rescale = c(0,1)
             X = rescale[2] * cbind(X, matrix(- 1/k, nrow = nrow(X), ncol = k))
-            bhat = c(bhat, object$zeta - rescale[1] / rescale[2])
+            ### bhat = c(bhat, object$zeta - rescale[1] / rescale[2])
             misc = list(offset.mult = rescale[2])
         }
         else {
-            bhat = c(bhat, object$zeta)
+            ### bhat = c(bhat, object$zeta)
             j = matrix(1, nrow=k, ncol=1)
             J = matrix(1, nrow=nrow(X), ncol=1)
             X = cbind(kronecker(-j, X), kronecker(diag(1,k), J))
@@ -574,21 +587,30 @@ emm_basis.stanreg = function(object, trms, xlev, grid, mode, rescale, ...) {
                 misc$postGridHook = ".clm.postGrid" # we probably need to adapt this
             }
         }
+        nms = colnames(X) = c(nms, names(object$zeta))
         
         misc$respName = as.character.default(terms(object))[2]
     }
-    samp = as.matrix(object$stanfit)[, names(bhat), drop = FALSE]
+    samp = as.matrix(object$stanfit)[, nms, drop = FALSE]
     attr(samp, "n.chains") = object$stanfit@sim$chains
+
+    bhat = apply(samp, 2, mean)
+    V = cov(samp)
+    
     # estimability...
     nbasis = estimability::all.estble
     all.nms = colnames(X)
-    if (length(names(bhat)) < (n <- length(all.nms))) {
+    if (length(nms) < length(all.nms)) {
+        if(is.null(contr <- object$contrasts))
+            contr = attr(model.matrix(object), "contrasts")
         coef = NA * X[1, ]
         coef[names(bhat)] = bhat
         bhat = coef
         mmat = model.matrix(trms, object$data, contrasts.arg = contr)
         nbasis = estimability::nonest.basis(mmat)
     }
+    
+    
     list(X = X, bhat = bhat, nbasis = nbasis, V = V, 
          dffun = function(k, dfargs) Inf, dfargs = list(), 
          misc = misc, post.beta = samp)
