@@ -229,8 +229,8 @@ trt.vs.ctrl.emmc = function(levs, ref = 1, reverse = FALSE,
                             exclude = integer(0), include, ...) {
     ref = .num.key(levs, ref)
     exclude = .get.excl(levs, exclude, include)
-    if ((min(ref) < 1) || (max(ref) > length(levs)))
-        stop("Reference levels are out of range")
+    if (length(ref) == 0 || (min(ref) < 1) || (max(ref) > length(levs)))
+        stop("In trt.vs.ctrl.emmc(), 'ref' levels are out of range", call. = FALSE)
     k = length(levs)
     cnm = ifelse(length(ref)==1, 
                  levs[ref], 
@@ -395,6 +395,8 @@ identity.emmc = function(levs, exclude = integer(0), include, ...) {
 ### utility to translate character keys to index keys
 #' @export
 .num.key = function(levs, key) {
+    if(!is.null(raw <- attr(levs, "raw")))
+        levs = raw
     orig.key = key
     if (is.character(key))
         key = match(key, levs)
