@@ -126,6 +126,8 @@ vcov.emmGrid = function(object, ...) {
 }
 
 
+
+
 # Method to alter contents of misc slot
 
 #' Update an \code{emmGrid} object
@@ -198,11 +200,6 @@ vcov.emmGrid = function(object, ...) {
 #' \item{\code{adjust}}{(\code{character)}) is the default for the \code{adjust}
 #' argument in \code{\link{summary.emmGrid}}.}
 #' 
-#' \item{\code{estType}}{(\code{character}) is the type of the estimate. It
-#' should match one of \samp{c("prediction", "contrast", "pairs")}. This is used
-#' along with \code{"adjust"} to determine appropriate adjustments to P values
-#' and confidence intervals.}
-#' 
 #' \item{\code{famSize}}{(integer) is the number of means involved in a family of
 #' inferences; used in Tukey adjustment}
 #' 
@@ -216,6 +213,8 @@ vcov.emmGrid = function(object, ...) {
 #' \item{\code{df}}{(numeric) overrides the default degrees of freedom with a
 #' specified single value.}
 #' 
+#' \item{\code{calc}}{(list) additional calculated columns. See \code{\link{summary.emmGrid}}.}
+#' 
 #' \item{\code{null}}{(numeric) null hypothesis for \code{summary} or
 #' \code{test} (taken to be zero if missing).}
 #' 
@@ -234,6 +233,16 @@ vcov.emmGrid = function(object, ...) {
 #' \code{"link"} (with synonyms \code{"lp"} and \code{"linear"}), or
 #' \code{"response"}.}
 #' 
+#' \item{\code{bias.adjust}, \code{frequentist}}{(character) These
+#' are used by \code{summary} if the value of these arguments are not specified.}
+#' 
+#' \item{\code{estType}}{(\code{character}) is used internally to determine 
+#' what \code{adjust} methods are appropriate. It should match one of 
+#' \samp{c("prediction", "contrast", "pairs")}. As an example of why this is needed,
+#' the Tukey adjustment should only be used for pairwise comparisons 
+#' (\code{estType = "pairs"}); if \code{estType} is some other string, Tukey
+#' adjustments are not allowed.}
+#' 
 #' \item{\code{avgd.over}}{(\code{character)} vector) are the names of the 
 #' variables whose levels are averaged over in obtaining marginal averages of 
 #' predictions, i.e., estimated marginal means. Changing this might produce a 
@@ -251,17 +260,47 @@ vcov.emmGrid = function(object, ...) {
 #' structure. See \dQuote{Recovering or overriding model information} in the
 #' documentation for \code{\link{ref_grid}}. The current nesting structure is
 #' displayed by \code{\link{str.emmGrid}}.}
-#' 
+#'
 #' \item{\code{levels}}{named \code{list} of new levels for the elements of the
 #' current \code{emmGrid}. The list name(s) are used as new variable names, and
 #' if needed, the list is expanded using \code{expand.grid}. These results replace
-#' current variable names and levels. This specification changes the \code{levels}, 
+#' current variable names and levels. This specification changes the \code{levels},
 #' \code{grid}, \code{roles}, and \code{misc} slots in the updated \code{emmGrid},
 #' and resets \code{pri.vars}, \code{by.vars}, \code{adjust}, \code{famSize},
-#' \code{avgd.over}, and \code{nesting}. 
+#' and \code{avgd.over}. In addition, if there is nesting of factors, that may be 
+#' altered; a warning is issued if it involves something other than mere name changes.
 #' \emph{Note:} All six letters of \code{levels} is needed in order to distinguish
 #' it from \code{level}.}
 #' 
+#' \item{\code{submodel}}{\code{formula} or \code{character} value specifying a 
+#' submodel (requires this feature being supported by underlying methods 
+#' for the model class). When specified, the \code{linfct} slot is replaced by 
+#' its aliases for the specified sub-model. Any factors in the sub-model that 
+#' do not appear in the model matrix are ignored, as are any interactions that 
+#' are not in the main model, and any factors associate with multivariate responses. 
+#' The estimates displayed are then computed as if 
+#' the sub-model had been fitted. (However, the standard errors will be based on the
+#' error variance(s) of the full model.) 
+#' \emph{Note:} The formula should refer only to predictor names, \emph{excluding} any
+#' function calls (such as \code{factor} or \code{poly}) that appear in the 
+#' original model formula. See the example.
+#' 
+#' The character values allowed should partially 
+#' match \code{"minimal"} or \code{"type2"}. With \code{"minimal"}, the sub-model
+#' is taken to be the one only involving the surviving factors in \code{object}
+#' (the ones averaged over being omitted). Specifying \code{"type2"} is the same as
+#' \code{"minimal"} except only the highest-order term in the submodel is retained,
+#' and all effects not containing it are orthogonalized-out. Thus, in a purely linear
+#' situation such as an \code{lm} model, the joint test
+#' of the modified object is in essence a type-2 test as in \code{car::Anova}.
+#' 
+#' For some objects such as generalized linear models, specifying \code{submodel}
+#' will typically not produce the same estimates or type-2 tests as would be
+#' obtained by actually fitting a separate model with those specifications.
+#' The reason is that those models are fitted by iterative-reweighting methods,
+#' whereas the \code{submodel} calculations preserve the final weights used in
+#' fitting the full model.}
+#' 
 #' \item{(any other slot name)}{If the name matches an element of
 #' \code{slotNames(object)} other than \code{levels}, that slot is replaced by 
 #' the supplied value, if it is of the required class (otherwise an error occurs). 
@@ -271,25 +310,39 @@ vcov.emmGrid = function(object, ...) {
 #' and dimensions of \code{grid}, \code{linfct}, \code{bhat}, and \code{V} must
 #' conform.}
 #' } %%%%%%% end \describe 
+#' 
+#' @note
+#' When it makes sense, an option set by \code{update} will persist into 
+#' future results based on that object. But some options are disabled as well.
+#' For example, a \code{calc} option will be nulled-out if \code{contrast}
+#' is called, because it probably will not make sense to do the same 
+#' calculations on the contrast results, and in fact the variable(s) needed
+#' may not even still exist.
+#' \code{factor(percent)}.
 #'
 #' @seealso \code{\link{emm_options}}
 #' @examples
 #' # Using an already-transformed response:
-#' mypigs <- transform(pigs, logconc = log(pigs$conc))
-#' mypigs.lm <- lm(logconc ~ source + factor(percent), data = mypigs)
+#' pigs.lm <- lm(log(conc) ~ source * factor(percent), data = pigs)
 #' 
-#' # Reference grid that knows about the transformation:
-#' mypigs.rg <- update(ref_grid(mypigs.lm), tran = "log", 
-#'                     predict.type = "response")
-#' emmeans(mypigs.rg, "source")
+#' # Reference grid that knows about the transformation
+#' # and asks to include the sample size in any summaries:
+#' pigs.rg <- update(ref_grid(pigs.lm), tran = "log", 
+#'                     predict.type = "response",
+#'                     calc = c(n = ~.wgt.))
+#' emmeans(pigs.rg, "source")
+#' 
+#' # Obtain estimates for the additive model
+#' # [Note that the submodel refers to 'percent', not 'factor(percent)']
+#' emmeans(pigs.rg, "source", submodel = ~ source + percent)
+#' 
+#' # Type II ANOVA
+#' joint_tests(pigs.rg, submodel = "type2")
 update.emmGrid = function(object, ..., silent = FALSE) {
     args = list(...)
-    valid.misc = c("adjust","alpha","avgd.over","by.vars","delta","df",
-                   "initMesg","estName","estType","famSize","infer","inv.lbl",
-                   "level","methDesc","nesting","null","predict.type","pri.vars"
-                   ,"side","sigma","tran","tran.mult","tran.offset","tran2","type","is.new.rg")
+    # see .valid.misc below this function for list of legal options
     valid.slots = slotNames(object)
-    valid.choices = union(valid.misc, valid.slots)
+    valid.choices = union(.valid.misc, valid.slots)
     misc = object@misc
     for (nm in names(args)) {
         fullname = try(match.arg(nm, valid.choices), silent=TRUE)
@@ -310,12 +363,20 @@ update.emmGrid = function(object, ..., silent = FALSE) {
                 grd = do.call(expand.grid, lvls)
                 if (nrow(object@grid) != nrow(grd))
                     stop("Length of replacement levels does not match the number of rows in the grid")
+                oldlvls = object@levels
+                if(!is.null(object@model.info$nesting) &&
+                   ((length(oldlvls) != length(lvls)) ||
+                    any(sapply(oldlvls, length) != sapply(lvls, length))))
+                    warning("Changes to levels may have altered nesting structure.\n",
+                            "You likely need to also run 'update(..., nesting = ...)'")
                 object@levels = lvls
                 for (nm in c(".wgt.", ".offset"))
                         grd[[nm]] = object@grid[[nm]]
                 object@grid = grd
                 object@roles$predictors = misc$pri.vars = names(lvls)
-                misc$by.vars = misc$avgd.over = object@model.info$nesting = NULL
+                misc$by.vars = misc$avgd.over = NULL
+                if (!is.null(object@model.info$nesting))
+                    object@model.info$nesting = .find_nests(grd, NULL, FALSE, lvls)
                 misc$adjust = "none"
                 misc$famSize = nrow(grd)
             }
@@ -340,6 +401,24 @@ update.emmGrid = function(object, ..., silent = FALSE) {
                         object@misc$display = .find.nonempty.nests(object, nms)
                     }
                 }
+                if (fullname == "submodel") {
+                    if(!is.null(A <- .alias.matrix(object, args[[nm]]))) {
+                        rcols = attr(A, "rcols")
+                        L = object@linfct
+                        k = ncol(L) / ncol(A)
+                        if(abs(k - (k<-as.integer(k))) > .01)
+                            stop("Incompatible columns in alias setup")
+                        ixmat = matrix(seq_along(L[1,]), ncol = k)
+                        for (j in seq_len(k)) {
+                            rc = ixmat[rcols, j]
+                            fc =ixmat[, j]
+                            object@linfct[, fc] = L[, rc, drop = FALSE] %*% A
+                        }
+                        object@model.info$model.matrix = "" # silent message
+                        misc$initMesg = c(misc$initMesg, paste("submodel: ~", attr(A, "submodstr")))
+                    }
+                }
+                    
                 else
                     misc[[fullname]] = args[[nm]]
             }
@@ -349,12 +428,26 @@ update.emmGrid = function(object, ..., silent = FALSE) {
     object
 }
 
+### List of valid strings to match in update() ###
+.valid.misc = c("adjust","alpha","avgd.over","bias.adjust","by.vars","calc","delta","df",
+               "initMesg","estName","estType","famSize","frequentist","infer","inv.lbl",
+               "level","methDesc","nesting","null","predict.type","pri.vars",
+               "side","sigma","tran","tran.mult","tran.offset","tran2","type","is.new.rg",
+               "submodel")
+
+
 #' Set or change emmeans options
 #'
 #' Use \code{emm_options} to set or change various options that are used in
 #' the \pkg{emmeans} package. These options are set separately for different contexts in
 #' which \code{emmGrid} objects are created, in a named list of option lists.
 #' 
+#' \pkg{emmeans}'s options are stored as a list in the system option \code{"emmeans"}. 
+#' Thus, \code{emm_options(foo = bar)} is the same as 
+#' \code{options(emmeans = list(..., foo = bar))} where \code{...} represents any
+#' previously existing options. The list \code{emm_defaults} contains the default
+#' values in case the corresponding element of system option \code{emmeans} is \code{NULL}.
+#' 
 #' Currently, the following main list entries are supported:
 #' \describe{
 #' \item{\code{ref_grid}}{A named \code{list} of defaults for objects created by
@@ -370,6 +463,16 @@ update.emmGrid = function(object, ..., silent = FALSE) {
 #'   \code{\link{summary.emmGrid}}, \code{\link{predict.emmGrid}}, \code{\link{test.emmGrid}},
 #'   \code{\link{confint.emmGrid}}, and \code{\link{emmip}}. The only option that can
 #'   affect the latter four is \code{"predict.method"}.}
+#' \item{\code{sep}}{A character value to use as a separator in labeling factor combinations.
+#'   Such labels are potentially used in several places such as \code{\link{contrast}} and 
+#'   \code{\link{plot.emmGrid}} when combinations of factors are compared or plotted.
+#'   The default is \code{" "}.}
+#' \item{\code{parens}}{Character vector that determines which labels are parenthesized
+#'   when they are contrasted. The first element is a regular expression, and the second and
+#'   third elements are used as left and right parentheses. 
+#'   See details for the \code{parens} argument in \code{\link{contrast}}. The default
+#'   will parenthesize labels containing the four arithmetic operators, 
+#'   using round parentheses.}
 #' \item{\code{cov.keep}}{The default value of \code{cov.keep} in \code{\link{ref_grid}}.
 #'   Defaults to \code{"2"}, i.e., two-level covariates are treated like factors.}
 #' \item{\code{graphics.engine}}{A character value matching 
@@ -400,6 +503,10 @@ update.emmGrid = function(object, ..., silent = FALSE) {
 #'   back transformation. If \code{TRUE} \emph{and \code{sigma} is available}, a
 #'   second-order adjustment is applied to estimate the mean on the response
 #'   scale.}
+#' \item{\code{enable.submodel}}{A logical value. If \code{TRUE}, enables support 
+#'   for selected model classes to implement the \code{submodel} option. If
+#'   \code{FALSE}, this support is disabled. Setting this option to \code{FALSE}
+#'   could save excess memory consumption.}
 #'   
 #' }%%% end describe{}
 #' Some other options have more specific purposes:
@@ -418,9 +525,32 @@ update.emmGrid = function(object, ..., silent = FALSE) {
 #' } %%%%%% end \describe
 #'
 #' @param ... Option names and values (see Details)
+#' @param disable If non-missing, this will reset all options to their defaults 
+#'   if \code{disable} tests \code{TRUE} (but first save them for possible later 
+#'   restoration). Otherwise, all previously saved options
+#'   are restored. This is important for bug reporting; please see the section below
+#'   on reproducible bugs. When \code{disable} is specified, the other arguments are ignored.
 #' 
 #' @return \code{emm_options} returns the current options (same as the result 
 #'   of \samp{getOption("emmeans")}) -- invisibly, unless called with no arguments.
+#' 
+#' @section Reproducible bugs:
+#' Most options set display attributes and such that are not likely to be associated
+#' with bugs in the code. However, some other options (e.g., \code{cov.keep})
+#' are essentially configuration settings that may affect how/whether the code
+#' runs, and the settings for these options may cause subtle effects that may be
+#' hard to reproduce. Therefore, when sending a bug report, please create a reproducible
+#' example and make sure the bug occurs with all options set at their defaults.
+#' This is done by preceding it with  \code{emm_options(disable = TRUE)}. 
+#' 
+#' By the way, \code{disable} works like a stack (LIFO buffer), in that \code{disable = TRUE}
+#' is equivalent to \code{emm_options(saved.opts = emm_options())} and 
+#' \code{emm_options(disable = FALSE)} is equivalent to 
+#' \code{options(emmeans = get_emm_option("saved.opts"))}. To completely erase
+#' all options, use \code{options(emmeans = NULL)}
+#' 
+#' 
+#' 
 #' @seealso \code{\link{update.emmGrid}}
 #' @export
 #' @examples
@@ -442,17 +572,35 @@ update.emmGrid = function(object, ..., silent = FALSE) {
 #' 
 #' # See tolerance being used for determining estimability
 #' get_emm_option("estble.tol")
+#' 
+#' \dontrun{
+#' # Set all options to their defaults
+#' emm_options(disable = TRUE)
+#' # ... and perhaps follow with code for a minimal reproducible bug,
+#' #     which may include emm_options() clls if they are pertinent ...
+#' 
+#' # restore options that had existed previously
+#' emm_options(disable = FALSE)
+#' }
 #'
-emm_options = function(...) {
+emm_options = function(..., disable) {
     opts = getOption("emmeans", list())
-    #    if (is.null(opts)) opts = list()
     newopts = list(...)
+    display = TRUE  # flag to display all options if ... is empty
+    if (!missing(disable)) {
+        if (disable)
+            opts = list(saved.opts = opts)
+        else if (!is.null(saved <- opts$saved.opts))
+            opts = saved
+        newopts = list()
+        display = FALSE
+    }
     for (nm in names(newopts))
         opts[[nm]] = newopts[[nm]]
     options(emmeans = opts)
     if (length(newopts) > 0)
         invisible(opts)
-    else {
+    else if (display) {
         opts = c(opts, emm_defaults)
         opts[sort(names(opts))]
     }
@@ -482,6 +630,8 @@ emm_defaults = list (
     contrast = list(infer = c(FALSE, TRUE)),
     save.ref_grid = TRUE,     # save new ref_grid in .Last.ref_grid
     cov.keep = "2",           # default for cov.keep arg in ref_grid
+    sep = " ",                # separator for combining factor levels
+    parens = c("-|\\+|\\/|\\*", "(", ")"), # patterns for what/how to parenthesize in contrast
     graphics.engine = "ggplot",  # default for emmip and plot.emmGrid
 ###    msg.data.call = TRUE,     # message when there's a call in data or subset
     msg.interaction = TRUE,   # message about averaging w/ interactions
@@ -490,6 +640,7 @@ emm_defaults = list (
     simplify.names = TRUE,    # simplify names like data$x to just "x"
     back.bias.adj = FALSE,    # Try to bias-adjust back-transformations?
     opt.digits = TRUE,        # optimize displayed digits?
+    enable.submodel = TRUE,   # enable saving extra info for submodel
     lmer.df = "kenward-roger",  # Use Kenward-Roger for df
     disable.pbkrtest = FALSE, # whether to bypass pbkrtest routines for lmerMod
     pbkrtest.limit = 3000,    # limit on N for enabling K-R
@@ -498,12 +649,163 @@ emm_defaults = list (
 )
 
 
+# override levels<- method
+#' @rdname update.emmGrid
+#' @export
+#' @param x an \code{emmGrid} object
+#' @param value \code{list} or replacement levels. See the documentation for
+#' \code{\link{update.emmGrid}} with the \code{levels} argument, 
+#' as well as the section below on \dQuote{Replaciong levels}
+#'   
+#' @return \code{levels<-} replaces the levels of the object in-place.
+#'   See the section on  for details.
+#' @section Replacing levels:
+#' The \code{levels<-} method uses \code{\link{update.emmGrid}} to replace the
+#' levels of one or more factors. This method allows selectively replacing
+#' the levels of just one factor (via subsetting operators), whereas 
+#' \code{update(x, levels = list(...))} requires a list of \emph{all} factors
+#' and their levels. If any factors are to be renamed, we must replace all
+#' levels and include the new names in the replacements. See the examples.
+#' 
+#' @examples
+#' 
+#' ## Changing levels of one factor
+#' newrg <- pigs.rg
+#' levels(newrg)$source <- 1:3
+#' newrg
+#' 
+#' ## Unraveling a previously standardized covariate
+#' zd = scale(fiber$diameter)
+#' fibz.lm <- lm(strength ~ machine * zd, data = fiber)
+#' (fibz.rg <- ref_grid(fibz.lm, at = list(zd = -2:2)))   ### 2*SD range
+#' lev <- levels(fibz.rg)
+#' levels(fibz.rg) <- list (
+#'     machine = lev$machine,
+#'     diameter = with(attributes(zd), 
+#'                     `scaled:center` + `scaled:scale` * lev$zd) )
+#' fibz.rg
+#' 
+"levels<-.emmGrid" = function(x, value) {
+    update.emmGrid(x, levels = value)
+}
+
+# ### transform method (I decided to ditch this completely in favor of levels<-)
+# #' Modify variable names and/or levels in a reference grid
+# #'
+# #' @param `_data` An object of class \code{emmGrid}
+# #' @param ... Specifications for changes to be made. See Specifications section
+# #' @param `_par` named \code{list} containing any additional parameters needed
+# #' in evaluating expressions
+# #' 
+# #' @section Specifications:
+# #' Each specification can be of one of the following forms:
+# #' \itemize{
+# #'   \item{\code{name = <replacement levels>  } (replace levels only)}
+# #'   \item{\code{name = newname ~ <replacement levels>  } (replace levels and rename)}
+# #'   \item{\code{name = ~ <expression>  } (calculate new levels)}
+# #'   \item{\code{name = newname ~ <expression>  } (calculate new levels and rename)}
+# #'   \item{\code{newame ~ name  } (rename with levels unchanged)}
+# #'   \item{\code{newname ~ <expression>  } (calculate new levels for variable 
+# #'          in expression, and rename)}
+# #' }
+# #' Here, \code{name} must be the name of an existing predictor in the grid,
+# #' and \code{<replacement levels>} is a character of numeric vector of length
+# #' exactly equal to the number of levels of \code{name}. The type of the replacement levels
+# #' does not need to match the type of the existing levels; however, any factor in the grid
+# #' remains a factor, with its levels changed.
+# #' 
+# #' Expressions must be supplied via a formula, and must be evaluable in the context 
+# #' of \code{envir} and the existing levels of \code{name}.
+# #' If a formula has a left-hand side, it is used as
+# #' a replacement name for that variable.
+# #'
+# #' @return a modified \code{emmGrid} object
+# #' @export
+# #' 
+# #' @note
+# #' An alternative way to use this is to supply a list of arguments as the \code{morph}
+# #' option in \code{\link{update.emmGrid}}.
+# #'
+# #' @examples
+# #' warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
+# #' (warp.rg <- ref_grid(warp.lm))
+# #' transform(warp.rg, tension = 1:3,  wool = texture ~ c("soft", "coarse"))
+# #' 
+# #' # Standardized predictor
+# #' z <- scale(fiber$diameter)
+# #' fiber.lm <- lm(strength ~ z + machine, data = fiber)
+# #' 
+# #' ### Mean predictions at 1-SD intervals:
+# #' (fiber.emm <- emmeans(fiber.lm, "z", at = list(z = -1:1)))
+# #' 
+# #' ### Same predictions labeled with actual diameter values:
+# #' transform(fiber.emm, diameter ~ `scaled:center` + `scaled:scale` * z,
+# #'     `_par` = attributes(z))
+# #' 
+# #' 
+# 
+# transform.emmGrid <- function(`_data`, ..., `_par` = list()) {
+#     specs = list(...)
+#     nms = names(c(`_dummy_` = 0, specs))[-1]  # keeps this from being NULL
+#     for (i in which(nms == "")) { # get name from formula rhs
+#         if (!inherits(spc <- specs[[i]], "formula") 
+#             || (inherits(spc, "formula") && (length(spc) < 3)))
+#             stop("Unnamed specifications must be two-sided formulas")
+#         nms[i] = names(specs)[i] = c(intersect(all.vars(spc[-2]),
+#                         names(`_data`@levels)), "(absent)")[1]
+#     }
+#     for (var in nms) {
+#         oldlev = `_data`@levels[[var]]
+#         if (is.null(oldlev))
+#             stop("No variable named '", var, "' in this object.")
+#         newlev = specs[[var]]
+#         if(inherits(newlev, "formula") && length(newlev) > 2) {
+#             newname = as.character(newlev)[2]
+#             newlev = newlev[-2]
+#         }
+#         else
+#             newname = ""
+#         if ((is.numeric(newlev) || is.character(newlev)) 
+#             && length(newlev) != length(oldlev))
+#             stop("Must provide exactly ", length(oldlev), " levels for '", var, "'")
+#         else if (inherits(newlev, "formula"))
+#             newlev = eval(str2expression(as.character(newlev)[2]), 
+#                           envir = c(`_data`@levels[var], `_par`))
+#         # so at this point we have conforming numbers of levels.
+#         `_data`@levels[[var]] = newlev
+#         v = newv = `_data`@grid[[var]]
+#         if (inherits(v, "factor"))
+#             levels(newv) = newlev
+#         else for (i in seq_along(oldlev)) 
+#             newv[v == oldlev[[i]]] = newlev[[i]]
+#         `_data`@grid[[var]] = newv
+#         if (newname != "") {
+#             i = which(names(`_data`@levels) == var)
+#             names(`_data`@levels)[i] = newname
+#             i = which(names(`_data`@grid) == var)
+#             names(`_data`@grid)[i] = newname
+#             i = which(`_data`@roles$predictors == var)
+#             if (length(i) > 0) 
+#                 `_data`@roles$predictors[i] = newname
+#             nst = `_data`@model.info$nesting
+#             if (!is.null(nst)) {
+#                 names(nst)[names(nst) == var] = newname
+#                 nst = lapply(nst, function(x) { x[x == var] = newname; x })
+#                 `_data`@model.info$nesting = nst
+#             }
+#         }
+#     }
+#     `_data`
+# }
+
+
+
 ### Utility to change the internal structure of an emmGrid object
 ### Returned emmGrid object has linfct = I and bhat = estimates
 ### Primary reason to do this is with transform = TRUE, then can 
 ### work with linear functions of the transformed predictions
 
-#' Reconstruct a reference grid with a new transformation or posterior sample
+#' Reconstruct a reference grid with a new transformation or simulations
 #' 
 #' The typical use of this function is to cause EMMs to be computed on
 #' a different scale, e.g., the back-transformed scale rather than the 
@@ -521,12 +823,13 @@ emm_defaults = list (
 #' contrasts will be conducted on the new scale -- which is
 #' the reason this function exists. 
 #' 
-#' This function may also be used to convert a reference grid for a 
-#' frequentist model to one for a Bayesian model. To do so, specify a value
-#' for \code{N.sim} and a posterior sample is simulated using the function \code{sim}.
-#' . The grid may be further processed in accordance with
-#' the other arguments; or if \code{transform = "pass"}, it is simply returned with the 
-#' only change being the addition of the posterior sample.
+#' This function may also be used to simulate a sample of regression
+#' coefficients for a frequentist model for subsequent use as though it were a
+#' Bayesian model. To do so, specify a value for \code{N.sim} and a sample is
+#' simulated using the function \code{sim}. The grid may be further processed in
+#' accordance with the other arguments; or if \code{transform = "pass"}, it is
+#' simply returned with the only change being the addition of the simulated
+#' sample.
 #' 
 #' @param object An object of class \code{emmGrid}
 #' @param transform Character, list, or logical value. If \code{"response"},
@@ -604,7 +907,7 @@ emm_defaults = list (
 #' # Same result, useful if we hadn't already created 'rg'
 #' # emm2 <- emmeans(pigs.lm, "source", transform = "response")
 #' 
-#' # Simulate a posterior sample
+#' # Simulate a sample of regression coefficients
 #' set.seed(2.71828)
 #' rgb <- regrid(rg, N.sim = 200, transform = "pass")
 #' emmeans(rgb, "source", type = "response")  ## similar to emm1
@@ -613,7 +916,7 @@ regrid = function(object, transform = c("response", "mu", "unlink", "none", "pas
                   bias.adjust = get_emm_option("back.bias.adj"), sigma, 
                   N.sim, sim = mvtnorm::rmvnorm, ...) 
 {
-    links = c("logit", "probit", "cauchit", "cloglog", "log", "sqrt", "1/mu^2", "inverse")
+    links = c("logit", "probit", "cauchit", "cloglog", "log", "log10", "log2", "sqrt", "1/mu^2", "inverse")
     if (is.logical(transform))   # for backward-compatibility
         transform = ifelse(transform, "response", "none")
     else if (is.list(transform)) {
@@ -624,7 +927,7 @@ regrid = function(object, transform = c("response", "mu", "unlink", "none", "pas
         transform = match.arg(transform)
     
     if (is.na(object@post.beta[1]) && !missing(N.sim)) {
-        message("Generating a posterior sample of size ", N.sim)
+        message("Simulating a sample of size ", N.sim, " of regression coefficients.")
         object@post.beta = sim(N.sim, object@bhat, object@V)
     }
 
@@ -711,7 +1014,7 @@ regrid = function(object, transform = c("response", "mu", "unlink", "none", "pas
     }
     if (transform %in% c(links, "user")) { # fake a transformation
         link = if (transform == "user") userlink
-               else                     make.link(transform)
+               else                     .make.link(transform)
         bounds = range(link$linkinv(c(-1e6, -100, -1, 0, 1, 100, 1e6)))
         nas = which(is.na(object@bhat)) # already NA
         incl = vincl = which((object@bhat > bounds[1]) & (object@bhat < bounds[2]))
@@ -748,6 +1051,7 @@ regrid = function(object, transform = c("response", "mu", "unlink", "none", "pas
     # Nix out things that are no longer needed or valid
     object@grid$.offset. = object@misc$offset.mult =
         object@misc$estHook = object@misc$vcovHook = NULL
+    object@model.info$model.matrix = "Submodels are not available with regridded objects"
     if(!missing(predict.type))
         object = update(object, predict.type = predict.type)
     object
