@@ -91,8 +91,8 @@
 recover_data = function(object, ...) {
     # look for outside methods first
     for (cl in .chk.cls(object)) {
-        rd <- try(getS3method("recover_data", cl, envir = .GlobalEnv), silent = TRUE)
-        if(!inherits(rd, "try-error"))
+        rd <- .get.outside.method("recover_data", cl)
+        if(!is.null(rd))
             return(rd(object, ...))
     }
     UseMethod("recover_data")
@@ -106,6 +106,17 @@ recover_data = function(object, ...) {
     setdiff(class(object)[1:2], sacred)
 }
 
+### My internal method dispatch -- we prefer outside methods
+.get.outside.method = function(generic, cls) {
+    mth = utils::getAnywhere(paste(generic, cls, sep = "."))
+    from = sapply(strsplit(mth[[3]], "[ :]"), function(x) rev(x)[1])
+    if (length(from) == 0)
+        return (NULL)
+    if(any(outside <- (from != "emmeans")))
+        mth[which(outside)[1]]
+    else
+        NULL
+}
 
 
 
@@ -134,13 +145,25 @@ recover_data = function(object, ...) {
 #'   a local variable \code{knots} that is not a predictor, but its value is
 #'   needed to fit the model. Names of parameters not actually used are harmless,
 #'   and the default value \code{"pi"} (the only numeric constant in base R)
-#'   is provided in case the model involves it.
+#'   is provided in case the model involves it. An example involving splines
+#'   may be found at \url{https://github.com/rvlenth/emmeans/issues/180}.
+#' @param frame Optional \code{data.frame}. Many model objects contain the 
+#'   model frame used when fitting the model. In cases where there are no 
+#'   predictor transformations, this model frame has all the original predictor
+#'   values and so is usable for recovering the data. Thus, if \code{frame} is
+#'   non-missing and \code{data} is \code{NULL}, a check is made on \code{trms}
+#'   and if there are no function calls, we use \code{data = frame}. This
+#'   can be helpful because it provides a modicum of security against the
+#'   possibility that the original data used when fitting the model has been
+#'   altered or removed.
 #' 
 #' @method recover_data call
 #' @export
-recover_data.call = function(object, trms, na.action, data = NULL, params = "pi", ...) {
+recover_data.call = function(object, trms, na.action, data = NULL, params = "pi", frame, ...) {
     fcall = object # because I'm easily confused
     vars = setdiff(.all.vars(trms), params)
+    if (!missing(frame) && is.null(data) && !.has.fcns(trms))
+        data = frame
     tbl = data
     if (length(vars) == 0 || vars[1] == "1") {
         tbl = data.frame(c(1,1))
@@ -307,11 +330,11 @@ recover_data.call = function(object, trms, na.action, data = NULL, params = "pi"
 emm_basis = function(object, trms, xlev, grid, ...) {
     # look for outside methods first
     for (cl in .chk.cls(object)) {
-        emb <- try(getS3method("emm_basis", cl, envir = .GlobalEnv), silent = TRUE)
-        if(!inherits(emb, "try-error"))
+        emb <- .get.outside.method("emm_basis", cl)
+        if(!is.null(emb))
             return(emb(object, trms, xlev, grid, ...))
     }
-    UseMethod("emm_basis")
+    UseMethod("emm_basis") # lands here only if no outside method found
 }
 
 # Hidden courtesy function that provides access to all recover_data methods
