@@ -27,10 +27,14 @@
 #' Summaries, predictions, intervals, and tests for \code{emmGrid} objects
 #' 
 #' These are the primary methods for obtaining numerical or tabular results 
-#' from an \code{emmGrid} object. Note that by default, summaries for Bayesian models are
+#' from an \code{emmGrid} object. 
+#' \code{summary.emmGrid} is the general function for summarizing \code{emmGrid} objects. 
+#' It also serves as the print method for these objects; so for convenience,
+#' \code{summary()} arguments may be included in calls to functions such as 
+#' \code{\link{emmeans}} and \code{\link{contrast}} that construct \code{emmGrid} 
+#' objects. Note that by default, summaries for Bayesian models are
 #' diverted to \code{\link{hpd.summary}}.
 #' 
-#' \code{summary.emmGrid} is the general function for summarizing \code{emmGrid} objects. 
 #' \code{confint.emmGrid} is equivalent to \code{summary.emmGrid with 
 #' infer = c(TRUE, FALSE)}. When called with \code{joint = FALSE}, \code{test.emmGrid}
 #' is equivalent to \code{summary.emmGrid} with \code{infer = c(FALSE, TRUE)}. 
@@ -71,6 +75,12 @@
 #' @param df Numeric. If non-missing, a constant number of degrees of freedom to
 #'   use in constructing confidence intervals and \emph{P} values (\code{NA}
 #'   specifies asymptotic results).
+#' @param calc Named list of character value(s) or formula(s).
+#'   The expressions in \code{char} are evaluated and appended to the
+#'   summary, just after the \code{df} column. The expression may include
+#'   any names up through \code{df} in the summary, any additional names in 
+#'   \code{object@grid} (such as \code{.wgt.} or \code{.offset.}), or any
+#'   earlier elements of \code{calc}.
 #' @param null Numeric. Null hypothesis value(s), on the linear-predictor scale,
 #'   against which estimates are tested. May be a single value used for all, or
 #'   a numeric vector of length equal to the number of tests in each family
@@ -99,8 +109,10 @@
 #'   \code{object@misc$sigma} is used, and an error is thrown if it is not found.
 #'   \emph{Note:} \code{sigma} may be a vector, as long as it conforms to the number of rows
 #'   of the reference grid.
-#' @param ... (Not used by \code{summary.emmGrid}.) In
-#'   \code{as.data.frame.emmGrid}, \code{confint.emmGrid}, \code{predict.emmGrid}, and 
+#' @param ... Optional arguments such as \code{scheffe.rank} 
+#'   (see \dQuote{P-value adjustments}). 
+#'   In \code{as.data.frame.emmGrid}, \code{confint.emmGrid}, 
+#'   \code{predict.emmGrid}, and 
 #'   \code{test.emmGrid}, these arguments are passed to
 #'   \code{summary.emmGrid}.
 #'
@@ -114,9 +126,10 @@
 #'   results of \code{as.mcmc}.
 #'   
 #' @section Defaults:
-#'   The \code{misc} slot in \code{object} contains default values for
-#'   \code{by}, \code{infer}, \code{level}, \code{adjust}, \code{type},
-#'   \code{null}, \code{side}, and \code{delta}. These defaults vary depending
+#'   The \code{misc} slot in \code{object} may contain default values for
+#'   \code{by}, \code{calc}, \code{infer}, \code{level}, \code{adjust}, 
+#'   \code{type}, \code{null}, \code{side}, and \code{delta}. 
+#'   These defaults vary depending
 #'   on the code that created the object. The \code{\link{update}} method may be
 #'   used to change these defaults. In addition, any options set using 
 #'   \samp{emm_options(summary = ...)} will trump those stored in the object's 
@@ -149,9 +162,15 @@
 #'     of means in the family. (Available for two-sided cases only.)}
 #'   \item{\code{"scheffe"}}{Computes \eqn{p} values from the \eqn{F}
 #'     distribution, according to the Scheffe critical value of
-#'     \eqn{\sqrt{kF(k,d)}}{sqrt[k*F(k,d)]}, where \eqn{d} is the error degrees of
-#'     freedom and \eqn{k} is (family size minus 1) for contrasts, and (number of
-#'     estimates) otherwise. (Available for two-sided cases only.)}
+#'     \eqn{\sqrt{rF(\alpha; r, d)}}{sqrt[r*qf(alpha, r, d)]}, where \eqn{d} is
+#'     the error degrees of freedom and \eqn{r} is the rank of the set of linear
+#'     functions under consideration. By default, the value of \code{r} is
+#'     computed from \code{object@linfct} for each by group; however, if the
+#'     user specifies an argument matching \code{scheffe.rank}, its value will
+#'     be used instead. Ordinarily, if there are \eqn{k} means involved, then
+#'     \eqn{r = k - 1} for a full set of contrasts involving all \eqn{k} means, and
+#'     \eqn{r = k} for the means themselves. (The Scheffe adjustment is available
+#'     for two-sided cases only.)}
 #'   \item{\code{"sidak"}}{Makes adjustments as if the estimates were independent
 #'     (a conservative adjustment in many cases).}
 #'   \item{\code{"bonferroni"}}{Multiplies \eqn{p} values, or divides significance
@@ -195,12 +214,21 @@
 #'   \code{\link{as.glht}} with methods in the \pkg{multcomp} package to obtain
 #'   non-conservative multi-step adjustments to tests.
 #'
-#' @section Testing nonsuperiority, noninferiority, or equivalence:
-#'   When \code{delta = 0}, test statistics are of the usual form 
-#'   \samp{(estimate - null)/SE}, or notationally, \eqn{t = (Q - \theta_0)/SE} 
-#'   where \eqn{Q} is our estimate of \eqn{\theta}; 
-#'   then left, right, or two-sided \eqn{p} values are produced.
-#' 
+#' @section Tests of significance, nonsuperiority, noninferiority, or equivalence:
+#'   When \code{delta = 0}, test statistics are the usual tests of significance.
+#'   They are of the form 
+#'   \samp{(estimate - null)/SE}. Notationally: 
+#'   \describe{
+#'     \item{Significance}{\eqn{H_0: \theta = \theta_0}  versus \cr
+#'        \eqn{H_1: \theta < \theta_0} (left-sided), or\cr
+#'       \eqn{H_1 \theta > \theta_0} (right-sided), or\cr
+#'       \eqn{H_1: \theta \ne \theta_0} (two-sided)\cr
+#'       The test statistic is\cr
+#'       \eqn{t = (Q - \theta_0)/SE}\cr 
+#'       where \eqn{Q} is our estimate of \eqn{\theta};
+#'       then left, right, or two-sided \eqn{p} values are produced, 
+#'       depending on \code{side}.}
+#'   }
 #'   When \code{delta} is positive, the test statistic depends on \code{side} as
 #'   follows.
 #'   \describe{
@@ -215,8 +243,12 @@
 #'   \item{Two-sided (equivalence)}{\eqn{H_0: |\theta - \theta_0| \ge \delta}
 #'     versus \eqn{H_1: |\theta - \theta_0| < \delta}\cr
 #'     \eqn{t = (|Q - \theta_0| - \delta)/SE}\cr
-#'     The \eqn{p} value is the \emph{lower}-tail probability.}
+#'     The \eqn{p} value is the \emph{lower}-tail probability.\cr
+#'     Note that \eqn{t} is the maximum of \eqn{t_{nonsup}} and \eqn{-t_{noninf}}. 
+#'     This is equivalent to choosing the less 
+#'     significant result in the two-one-sided-test (TOST) procedure.}
 #'   } %%%%%%%%%%%% end \describe{}
+#'
 #' 
 #' @section Non-estimable cases:
 #'   When the model is rank-deficient, each row \code{x} of \code{object}'s
@@ -253,10 +285,11 @@
 #'   unnecessary to call \code{summary} unless there is a need to
 #'   specify other than its default options.
 #'   
-#' @seealso \code{link{hpd.summary}}
+#' @seealso \code{\link{hpd.summary}}
 #' 
 #' @method summary emmGrid  
 #' @export
+#' @order 1
 #'
 #' @examples
 #' warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
@@ -269,22 +302,38 @@
 #' pigs.lm <- lm(log(conc) ~ source + factor(percent), data = pigs)
 #' pigs.emm <- emmeans(pigs.lm, "percent", type = "response")
 #' summary(pigs.emm)    # (inherits type = "response")
+#' summary(pigs.emm, calc = c(n = ".wgt."))  # Show sample size
 #' 
 #' # For which percents is EMM non-inferior to 35, based on a 10% threshold?
 #' # Note the test is done on the log scale even though we have type = "response"
 #' test(pigs.emm, null = log(35), delta = log(1.10), side = ">")
-#'
-#' test(contrast(pigs.emm, "consec"))
 #' 
-#' test(contrast(pigs.emm, "consec"), joint = TRUE)
+#' con <- contrast(pigs.emm, "consec")
+#' test(con)
+#' 
+#' test(con, joint = TRUE)
+#' 
+#' # default Scheffe adjustment - rank = 3
+#' summary(con, infer = c(TRUE, TRUE), adjust = "scheffe")
+#' 
+#' # Consider as some of many possible contrasts among the six cell means
+#' summary(con, infer = c(TRUE, TRUE), adjust = "scheffe", scheffe.rank = 5)
 #'
-summary.emmGrid <- function(object, infer, level, adjust, by, type, df, 
+summary.emmGrid <- function(object, infer, level, adjust, by, type, df, calc,
                         null, delta, side, frequentist, 
                         bias.adjust = get_emm_option("back.bias.adj"),
                         sigma, ...) {
     if(missing(sigma))
         sigma = object@misc$sigma
-
+    if(missing(frequentist) && !is.null(object@misc$frequentist))
+        frequentist = object@misc$frequentist
+    if(missing(bias.adjust)) {
+        if (!is.null(object@misc$bias.adjust)) 
+            bias.adjust = object@misc$bias.adjust
+        else
+            bias.adjust = get_emm_option("back.bias.adj")
+    }
+    
     if(!is.na(object@post.beta[1]) && (missing(frequentist) || !frequentist))
         return (hpd.summary(object, prob = level, by = by, type = type, 
                             bias.adjust = bias.adjust, sigma = sigma, ...))
@@ -429,10 +478,25 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df,
         attr(corrmat, "by.rows") = by.rows
     }
     else if (!is.na(pmatch(adjust, "scheffe"))) {
-        sch.rank = sapply(by.rows, function(.) qr(object@linfct[., , drop = FALSE])$rank)
+        if(is.null(sch.rank <- .match.dots("scheffe.rank", ...)))
+            sch.rank = sapply(by.rows, function(.) qr(zapsmall(object@linfct[., , drop = FALSE]))$rank)
         if(length(unique(sch.rank)) > 1)
             fam.info[1] = "uneven"   # This forces ragged.by = TRUE in .adj functions
     }
+    
+    # Add calculated columns
+    if(!missing(calc) || !is.null(calc <- misc$calc)) {
+        env = c(result, grid[setdiff(names(grid), names(result))])
+        for (v in names(calc)) {
+            elt = rev(as.character(calc[[v]]))[1] # pick out rhs if a formula
+            val = try(eval(parse(text = elt), envir = env), silent = TRUE)
+            if(!inherits(val, "try-error"))
+                result[[v]] = env[[v]] = val
+            else
+                warning("The column '", v, "' could not be calculated, ",
+                " so it is omitted")
+        }
+    }
 
     if(infer[1]) { # add CIs
         acv = .adj.critval(level, result$df, adjust, fam.info, side, corrmat, by.rows, sch.rank)
@@ -509,7 +573,7 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df,
     attr(summ, "clNames") = cnm  # will be NULL if infer[1] is FALSE
     if (is.null(misc$pri.vars) || length(misc$pri.vars) == 0)
         misc$pri.vars = names(object@levels)
-    attr(summ, "pri.vars") = setdiff(union(misc$pri.vars, misc$by.vars), by)
+    attr(summ, "pri.vars") = setdiff(union(misc$pri.vars, misc$by.vars), c(by, ".wgt.", ".offset."))
     attr(summ, "by.vars") = by
     attr(summ, "adjust") = adjust
     attr(summ, "side") = side
@@ -523,6 +587,7 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df,
 # S3 predict method
 
 #' @rdname summary.emmGrid
+#' @order 4
 #' @method predict emmGrid
 #' @param interval Type of interval desired (partial matching is allowed): 
 #' \code{"none"} for no intervals,
@@ -583,6 +648,7 @@ predict.emmGrid <- function(object, type,
 # as.data.frame method
 
 #' @rdname summary.emmGrid
+#' @order 5
 #' @param x object of the given class
 #' @param row.names passed to \code{\link{as.data.frame}}
 #' @param optional passed to \code{\link{as.data.frame}}
@@ -596,6 +662,7 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
 
 
 #' @rdname summary.emmGrid
+#' @order 6
 #' @method [ summary_emm
 #' @param as.df Logical value. With \code{x[..., as.df = TRUE]}, the result is
 #'   object is coerced to an ordinary \code{\link{data.frame}}; otherwise, it is left as a 
@@ -720,6 +787,13 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
 ####!!!!! TODO: Re-think whether we are handling Scheffe adjustments correctly
 ####!!!!!       if/when we shift around 'by' specs, etc.
 
+### utility for changing adjustments
+.chg.adjust = function(old, new, reason) {
+    message("Note: adjust = \"", old, "\" was changed to \"", new, 
+            "\"\nbecause \"", old, "\" is ", reason)
+    new
+}
+
 # utility to compute an adjusted p value
 # tail is -1, 0, 1 for left, two-sided, or right
 # Note fam.info is c(famsize, ncontr, estTypeIndex)
@@ -741,9 +815,9 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
         stop("Adjust method '", adjust, "' is not recognized or not valid")
     adjust = adj.meths[k]
     if ((tail != 0) && (adjust %in% c("tukey", "scheffe", "dunnettx"))) # meth not approp for 1-sided
-        adjust = "sidak"
+        adjust = .chg.adjust(adjust, "sidak", "not appropriate for one-sided inferences")
     if ((et != 3) && adjust == "tukey") # not pairwise
-        adjust = "sidak"
+        adjust = .chg.adjust(adjust, "sidak", "only appropriate for one set of pairwise comparisons")
     
     ragged.by = (is.character(fam.size) || adjust %in% c("mvt", p.adjust.methods))   # flag that we need to do groups separately
     if (!ragged.by)
@@ -807,7 +881,7 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
 #         else             paste(n.contr, "tests")
         xtra = switch(adjust, 
                       tukey = paste("for comparing a family of", fam.size, "estimates"),
-                      scheffe = paste("with dimensionality", scheffe.dim),
+                      scheffe = paste("with rank", scheffe.dim),
                       paste("for", n.contr, "tests")
                 )
         mesg = paste("P value adjustment:", adjust, "method", xtra)
@@ -841,12 +915,10 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
     if (!ragged.by && (adjust != "mvt") && (length(unique(DF)) == 1))
         by.rows = list(seq_along(DF))       # not ragged, we can do all as one by group
     
-    if ((et != 3) && adjust == "tukey") # not pairwise
-        adjust = "sidak"
     if ((tail != 0) && (adjust %in% c("tukey", "scheffe", "dunnettx"))) # meth not approp for 1-sided
-        adjust = "sidak"
+        adjust = .chg.adjust(adjust, "sidak", "not appropriate for one-sided inferences")
     if ((et != 3) && adjust == "tukey") # not pairwise
-        adjust = "sidak"
+        adjust = .chg.adjust(adjust, "sidak", "only appropriate for one set of pairwise comparisons")
     
     # asymptotic results when df is NA
     DF[is.na(DF)] = Inf
@@ -871,7 +943,7 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
 #        else             paste(n.contr, "estimates")
         xtra = switch(adjust, 
                       tukey = paste("for comparing a family of", fam.size, "estimates"),
-                      scheffe = paste("with dimensionality", scheffe.dim),
+                      scheffe = paste("with rank", scheffe.dim),
                       paste("for", n.contr, "estimates")
         )
         mesg = paste("Conf-level adjustment:", adjust, "method", xtra)
